# Tai Sakuma <tai.sakuma@gmail.com>
import collections
import copy

##__________________________________________________________________||
class atdict(object):
    """An attribute-access ordered dictionary

    An `atdict` can be initialized with 1) another `atdict` or 2) any
    arguments that can initialize `collections.OrderedDict`.

    """

    def __init__(self, *args, **kwargs):

        try:
            # First, if `args[0]` is the only argument, assume it is
            # another `atdict`, and try to copy its contents.
            if kwargs or len(args) != 1:
                raise TypeError
            attrdict = collections.OrderedDict(args[0]._attrdict)
        except:
            # Otherwise, all arguments are simply given to
            # `OrderedDict` so that an `atdict` can be initialized
            # with any arguments that can initialize `OrderedDict`.
            attrdict = collections.OrderedDict(*args, **kwargs)

        super(atdict, self).__setattr__('_attrdict', attrdict)
        # self._attrdict = attrdict # this would cause infinite
                                    # recursion as __setattr__() is
                                    # implemented

    def __copy__(self):
        return self.__class__(self)

    def __deepcopy__(self, memo):
        ret = self.__class__()
        super(atdict, ret).__setattr__('_attrdict', copy.deepcopy(self._attrdict))
        return ret

    def __repr__(self):
        return '{}({})'.format(
            self.__class__.__name__,
            ', '.join(['{}={!r}'.format(k, v) for k, v in self._attrdict.items()])
        )

    def __getattr__(self, attr):
        try:
            return self._attrdict[attr]
        except KeyError:
            raise AttributeError('{} has no attribute "{}"'.format(self, attr))

    def __setattr__(self, name, value):
        self._attrdict[name] = value

    def __delattr__(self, attr):
        del self._attrdict[attr]

    def __getstate__(self):
        return self.__dict__

    def __setstate__(self, dict):
        super(atdict, self).__setattr__('__dict__', dict)

    def __eq__(self, other):
        return self._attrdict == other._attrdict

##__________________________________________________________________||



from setuptools import setup, find_packages
import versioneer

from pathlib import Path

here = Path(__file__).resolve().parent
long_description = here.joinpath('README.md').read_text()

setup(
    name='atdict',
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description='Attribute-access ordered dictionary',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Tai Sakuma',
    author_email='tai.sakuma@gmail.com',
    url='https://github.com/alphatwirl/atdict',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    packages=find_packages(exclude=['docs', 'tests']),
    extras_require={
        'tests': [
            'pytest>=6.2',
            'pytest-cov>=2.11'
        ]
    }
)
